<?php

/**
 * Plugin Name: Spam Sweeper
 * Plugin URI: https://wordpress.org/plugins/spam-sweeper/
 * Description: Automatically delete spam comments so you can focus on blogging.
 * Author: Daniel James
 * Author URI: https://danieltj.uk/
 * Text Domain: spam-sweeper
 * Version: 1.3
 */

/**
 * (c) Copyright 2019, Daniel James
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

if ( ! defined( 'ABSPATH' ) ) {
	die();
}

$Spam_Sweeper = new Spam_Sweeper;

class Spam_Sweeper {

	/**
	 * The plugin version.
	 * 
	 * @since 1.0
	 * 
	 * @var string
	 */
	protected static $version = '1.3';

	/**
	 * Sweep the hooks away.
	 * 
	 * @since 1.0
	 * 
	 * @return void
	 */
	public function __construct() {

		register_activation_hook( __FILE__, array( __CLASS__, 'plugin_install' ), 10, 0 );
		register_deactivation_hook( __FILE__, array( __CLASS__, 'plugin_uninstall' ), 10, 0 );

		add_action( 'plugins_loaded', array( __CLASS__, 'load_text_domain' ), 10, 0 );
		add_action( 'init', array( __CLASS__, 'clear_the_spam' ), 10, 0 );
		add_action( 'admin_menu', array( __CLASS__, 'add_admin_pages' ), 10, 0 );
		add_action( 'admin_init', array( __CLASS__, 'register_settings' ), 10, 0 );
		add_action( 'updated_option', array( __CLASS__, 'saved_so_reset_timer' ), 10, 1 );

		add_filter( 'plugin_action_links', array( __CLASS__, 'add_donate_link' ), 10, 2 );

	}

	/**
	 * Install the plugin.
	 * 
	 * @since 1.0
	 * 
	 * @return void
	 */
	public static function plugin_install() {

		// Add the plugin options
		add_option( 'spam_sweeper_next_time', date( 'Y-m-d H:i:s', strtotime( '+1 day' ) ) );
		add_option( 'spam_sweeper_frequency', 'daily' );
		add_option( 'spam_sweeper_trash_too', 'off' );

	}

	/**
	 * Uninstall the plugin.
	 * 
	 * @since 1.0
	 * 
	 * @return void
	 */
	public static function plugin_uninstall() {

		// Delete the options
		delete_option( 'spam_sweeper_next_time' );
		delete_option( 'spam_sweeper_frequency' );
		delete_option( 'spam_sweeper_trash_too' );

	}

	/**
	 * Load the plugin text domain.
	 * 
	 * @since 1.0
	 * 
	 * @return void
	 */
	public static function load_text_domain() {

		load_plugin_textdomain( 'spam-sweeper', false, untrailingslashit( dirname( __FILE__ ) ) . '/languages' );

	}

	/**
	 * Adds a donate link to the plugins table.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated the l10n functions.
	 * 
	 * @param array  $links A list of plugin links
	 * @param string $file  The current plugin file.
	 * 
	 * @return array $links
	 */
	public static function add_donate_link( $links, $file ) {

		// Check if this is the current plugin
		if ( $file == 'spam-sweeper/spam-sweeper.php' ) {

			// Create the donate link
			$donate_link = '<a href="https://www.paypal.me/dtj27" target="_blank">' . esc_html__('Donate', 'spam-sweeper') . '</a>';

			// Add the link to the array
			array_unshift( $links, $donate_link );

		}

		return $links;

	}

	/**
	 * Sets the event timer.
	 * 
	 * @since 1.0
	 * 
	 * @return string|boolean New time string or false on failure.
	 */
	public static function reset_event() {

		$frequency = get_option( 'spam_sweeper_frequency', false );

		// Check the frequency value
		if ( 'hourly' != $frequency && 'daily' != $frequency && 'weekly' != $frequency ) {

			// Default to daily
			$frequency = 'daily';

		}

		// Get the increment value
		if ( 'hourly' == $frequency ) {

			$increment = '+1 hour';

		} elseif ( 'daily' == $frequency ) {

			$increment = '+1 day';

		} elseif ( 'weekly' == $frequency ) {

			$increment = '+1 week';

		}

		// Set the next event date
		$next_time = date( 'Y-m-d H:i:s', strtotime( $increment . ' ' . date( 'Y-m-d H:i:s' ) ) );

		// Update the event time
		$update = update_option( 'spam_sweeper_next_time', $next_time );

		/**
		 * Fires when the timer has been reset.
		 * 
		 * @since 1.1
		 * @since 1.2 Added the updated option value.
		 * 
		 * @param string|boolean $update New time string or false.
		 */
		do_action( 'after_spam_timer_reset', $update );

		return $update;

	}

	/**
	 * Event that clears out the spam.
	 * 
	 * @since 1.0
	 * 
	 * @return void
	 */
	public static function clear_the_spam() {

		// Get the event time
		$next_time = get_option( 'spam_sweeper_next_time', false );

		// Get the time now
		$current_time = date( 'Y-m-d H:i:s' );

		/**
		 * Filter to prevent the clean out running.
		 * 
		 * @since 1.0
		 * 
		 * @param boolean Flag to allow the event to run.
		 * 
		 * @return boolean
		 */
		$can_run = apply_filters( 'pre_spam_clear_event', true );

		// Is the event triggered?
		if ( true !== $can_run || false === $next_time || $next_time > $current_time ) {

			return false;

		}

		$where_clause = "comment_approved IN('spam')";

		// Check if we should empty the trash
		if ( 'on' == get_option( 'spam_sweeper_trash_too' ) ) {

			// Include the trash
			$where_clause = "comment_approved IN('spam', 'trash')";

		}

		global $wpdb;

		// Delete spam (and maybe trash)
		$query = $wpdb->query(
			'DELETE FROM ' . $wpdb->prefix . 'comments WHERE ' . $where_clause
		);

		$reset = self::reset_event();

		/**
		 * Fires after the delete query has been run.
		 * 
		 * @since 1.0
		 * @since 1.2 Added the reset option value.
		 * 
		 * @param string|boolean $reset New time string or false.
		 */
		do_action( 'after_spam_clear_event', $reset );

	}

	/**
	 * Add the plugin page.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated l10n functions and text strings.
	 * 
	 * @return void
	 */
	public static function add_admin_pages() {

		add_submenu_page(
			'tools.php',
			esc_html__('Spam Sweeper', 'spam-sweeper'),
			esc_html__('Spam', 'spam-sweeper'),
			'manage_options',
			'spam-sweeper',
			array(
				__CLASS__,
				'show_plugin_page'
			)
		);

	}

	/**
	 * Show the plugin page.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated l10n functions.
	 * 
	 * @return mixed
	 */
	public static function show_plugin_page() {

		// Get the next scheduled time
		$next_time = get_option( 'spam_sweeper_next_time', false );
		$next_time = date( 'jS F Y, H:i', strtotime( $next_time ) );

		?>

			<div class="wrap">
				<h1 class="page-title"><?php esc_html_e('Spam Sweeper', 'spam-sweeper'); ?></h1>
				<?php if ( false !== $next_time ) : ?>
					<p><?php printf( esc_html__('The next scheduled time to delete spam is %s.', 'spam-sweeper'), '<strong>' . $next_time . '</strong>' ); ?></p>
				<?php endif; ?>
				<div id="settings" class="tab-content">
					<?php settings_errors(); ?>
					<form method="post" action="options.php">
						<table class="plugin-form form-table">
							<tbody>
								<?php settings_fields('spam_sweeper_fields'); ?>
								<?php do_settings_sections('spam_sweeper_section'); ?>
							</tbody>
						</table>
						<?php submit_button( esc_html__('Save Settings', 'spam-sweeper'), 'primary' ); ?>
					</form>
				</div>
			</div>

		<?php

	}

	/**
	 * Register the plugin settings.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated l10n functions.
	 * 
	 * @return void
	 */
	public static function register_settings() {

		add_settings_section( 'spam_sweeper_group', false, false, 'spam_sweeper_section' );

		add_settings_field( 'spam_sweeper_frequency', esc_html__('Delete Frequency', 'spam-sweeper'), array( __CLASS__, 'spam_sweeper_frequency_template' ), 'spam_sweeper_section', 'spam_sweeper_group', array( 'spam_sweeper_frequency' ) );
		add_settings_field( 'spam_sweeper_trash_too', esc_html__('Clear Trash', 'spam-sweeper'), array( __CLASS__, 'spam_sweeper_trash_too_template' ), 'spam_sweeper_section', 'spam_sweeper_group', array( 'spam_sweeper_trash_too' ) );

		register_setting( 'spam_sweeper_fields', 'spam_sweeper_frequency', 'esc_attr');
		register_setting( 'spam_sweeper_fields', 'spam_sweeper_trash_too', 'esc_attr');

	}

	/**
	 * Show the frequency setting HTML.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated l10n functions.
	 * 
	 * @return mixed
	 */
	public static function spam_sweeper_frequency_template( $args ) {

		$settings_id = $args[0];

		?>

			<p>
				<select name="<?php echo $settings_id; ?>" id="<?php echo $settings_id; ?>" aria-describedby="description-<?php echo $settings_id; ?>">
					<option value="hourly"<?php if ( 'hourly' == get_option( $settings_id ) ) : ?> selected="selected"<?php endif; ?>><?php esc_html_e('Hourly', 'spam-sweeper'); ?></option>
					<option value="daily"<?php if ( 'daily' == get_option( $settings_id ) ) : ?> selected="selected"<?php endif; ?>><?php esc_html_e('Daily', 'spam-sweeper'); ?></option>
					<option value="weekly"<?php if ( 'weekly' == get_option( $settings_id ) ) : ?> selected="selected"<?php endif; ?>><?php esc_html_e('Weekly', 'spam-sweeper'); ?></option>
				</select>
			</p>
			<p class="description" id="description-<?php echo $settings_id; ?>"><?php esc_html_e('Select how frequently comments should be deleted.', 'spam-sweeper'); ?></p>

		<?php

	}

	/**
	 * Show the frequency setting HTML.
	 * 
	 * @since 1.0
	 * @since 1.1 Updated l10n functions.
	 * 
	 * @return mixed
	 */
	public static function spam_sweeper_trash_too_template( $args ) {

		$settings_id = $args[0];

		?>

			<p>
				<input type="checkbox" name="<?php echo $settings_id; ?>" id="<?php echo $settings_id; ?>" aria-describedby="description-<?php echo $settings_id; ?>"<?php if ( 'on' == get_option( $settings_id ) ) : ?> checked="checked"<?php endif; ?> />
			</p>
			<p class="description" id="description-<?php echo $settings_id; ?>"><?php esc_html_e('Should comments in the trash also be deleted?', 'spam-sweeper'); ?></p>

		<?php

	}

	/**
	 * Reset the timer if the frequency option is updated.
	 * 
	 * @since 1.1
	 * 
	 * @param string $option The name of the option.
	 * 
	 * @return void
	 */
	public static function saved_so_reset_timer( $option ) {

		// Check the frequency option was updated
		if ( 'spam_sweeper_frequency' == $option ) {

			$reset = self::reset_event();

		}

	}

}

